# DMSOCIO: SCRIPTS R PARA ANALISES SOCIOLINGUISTICAS - v. 0.1.4 (jan/2014)
# L. Oushiro (livia.oushiro@usp.br)
# Use! Modifique! Redistribua!

          ##################
          #    MODELOS     #
          ##################

#Para usar as funcoes do pacote dmsocio, execute a linha de comando abaixo apos iniciar cada sessao no R.
#Substitua "~" pelo caminho completo do diretorio em que se encontra o arquivo dmsocio.r.
source("~/dmsocio.R")

#Com conexao aa Internet, voce pode alternativamente executar a linha de comando abaixo para usar o script mais atualizado:
source("http://tinyurl.com/dmsocio") 

##########################
# Funcao identificacao() #
##########################

#O que faz: Realiza busca em um conjunto de transcricoes por um padrao 
#           e retorna arquivo(s) de texto com a marcacao das ocorrencias.

#Especifique o diretorio padrao - setwd() - como a pasta em que estao as transcricoes

setwd("~")

identificacao(padrao="",
              simbolo.marcacao="<>",
              posicao.marcacao=T, 
              ignorar.linhas=NULL,
              stoplist=NULL,
              novos.arquivos="marcacoes"
              )

# Argumentos da funcao identificacao()

# padrao            Obrigatorio. Definicao da sequencia de caracteres 
#                   que identifica as variantes da variavel.
#                   Nota: Deve ser especificado entre aspas.
# simbolo.marcacao  Opcional. Definicao dos simbolos que serao usados 
#                   para identificar as ocorrencias da variavel no corpus.
#                   Default = "<>".
# posicao.marcacao  Obrigatorio. Logico (T ou F). 
#                   Se T, a marcacao e colocada apos o padrao. 
#                   Se F, a marcacao e colocada antes do padrao.
#                   Default=T
#                   Nota: Para posicao.marcacao=F, a definicao da variavel preferivalmente deve comecar com \\b (levando em conta o inicio da palavra)
# ignorar.linhas    Opcional. Vetor com identificacao dos participantes 
#                   cujas falas devem ser ignoradas na busca do padrao.
#                   Default=NULL.          
# stoplist          Opcional. Vetor com palavras que devem ignoradas na busca do padrao. 
#                   Default=NULL.          
# novos.arquivos    Opcional. Sequencia de caracteres a ser adicionada ao nome dos arquivos originais,
#                   para diferenciar os arquivos com marcacoes de ocorrencia na mesma pasta.
#                   Nota: Deve ser especificado entre aspas.
#                   Default="marcacoes".          
          
#####################
# Funcao extracao() #
#####################

#O que faz: Realiza busca em um conjunto de transcricoes com marcacoes de ocorrencia 
#          e retorna uma tabela com a ocorrencia, X palavras do contexto precedente, X palavras do contexto seguinte, 
#          e localizacao das ocorrencias (paragrafo do arquivo original). 
#          Opcionalmente, retorna codificacao da variavel dependente e codificacao dos grupos de fatores sociais. 
          
#Requer definicao do diretorio padrao - setwd() - como a pasta em que estao as transcricoes
          
setwd("~")

extracao(padrao="",
         palavras.cont.precedente=5,
         palavras.ocorrencia=1,
         palavras.cont.seguinte=5,
         stoplist=NULL,
         ignorar.linhas=NULL,
         var.dependente=NULL,
         loc.variaveis.sociais=NULL, 
         nomes.colunas.variaveis=NULL,
         file="DadosExtraidos.txt"
         )

# Argumentos para a funcao extracao()

# padrao                    Obrigatorio. Definicao da sequencia de caracteres 
#                           a ser buscada.
#                           Nota: Deve ser especificado entre aspas.
# palavras.cont.precedente  Opcional. Numerico. Numero de palavras do contexto 
#                           precedente a serem extraidas.
#                           Default=5.
# palavras.ocorrencia       Opcional. Numerico. Numero de palavras a serem extraidas
#                           para a coluna de ocorrencia.
#                           Default=1.
# palavras.cont.seguinte    Opcional. Numerico. Numero de palavras do contexto
#                           seguinte a serem extraidas.
#                           Default=5.
# stoplist                  Opcional. Vetor com palavras que devem ignoradas na busca do padrao. 
#                           Default=NULL.          
# ignorar.linhas            Opcional. Vetor com identificacao dos participantes 
#                           cujas falas devem ser ignoradas na busca do padrao.
#                           Default=NULL.          
# var.dependente            Opcional. Numerico. Localizacao do caracter de codificacao
#                           dentro do termo que contem o padrao buscado.
#                           Default=NULL.
# loc.variaveis.sociais     Opcional. Vetor com o numero das linhas em que se encontram
#                           as informacoes das variaveis sociais a serem extraidas.
#                           Default=NULL.
# nomes.colunas.variaveis   Opcional. Vetor com o nome das variaveis sociais a serem extraidas,
#                           na mesma sequencia do vetor com sua localizacao.  
#                           Default=NULL.
# file                      Opcional. Nome do arquivo com dados extraidos.
#                           Default="DadosExtraidos.txt".
#                           Nota: Deve conter a extensao .txt e ser especificado
#                           entre aspas.

          
#######################
# Funcao amostragem() #
#######################

#O que faz: Le arquivo de codificacao e retorna uma nova tabela com uma amostra do arquivo completo, 
#          de acordo com numero de dados e variavel (coluna de referencia) especificados.
          
#Requer dataframe com arquivo de codificacao completo.

setwd("~") 
dados<-read.table(file=choose.files(),header=T,sep="\t",quote="",na.strings="NA",comment.char="")

amostragem(numero.dados=50,
           coluna.referencia,
           data,
           novo.arquivo="DadosAmostrados.txt"
           )

# Argumentos para funcao amostragem()

# numero.dados      Opcional. Numerico. Numero de dados a serem selecionados 
#                   aleatoriamente da coluna de referencia.
#                   Default = 50.
# coluna.referencia Obrigatorio. Numerico. Coluna que deve ser usada como referencia 
#                   para amostragem.
#                   A = 1, B = 2, C = 3 etc.
# data              Obrigatorio. Dataframe com planilha de dados a serem amostrados.
# novo.arquivo      Opcional. Nome do novo arquivo com os dados amostrados.
#                   Default="DadosAmostrados.txt"

          
          
          
          
          
          
###################################################
# Script Tratamento de Dados - Ver Oushiro (2014) #
###################################################

#(1)
FabianaB<-scan(file=choose.files(),what="char",sep="\n")
FabianaB.limpo<-gsub("\\(|\\)","",FabianaB)
cat(FabianaB.limpo,file="FabianaB-limpo.txt",sep="\n")

#(3)
2 + 3 

#(5)
x <- 2 + 3 

#(6)
x 

#(7)
x + 10 

#(8)
x <- (16-4)*2 
#(9)
rm(x) 

#(10)
y <- "gato" 

#(12)
variaveis.sociais<-c("sexo.genero", "faixa.etaria", "escolaridade")
variaveis.sociais

#(13)
setwd("~/dmsocio-SP2010")

#(15)
dadosR<-read.table(file=choose.files(), header=T, sep="\t", quote="", comment.char="")

#(17)
FabianaB<-scan(file=choose.files(),what="char",sep="\n")
FabianaB 

#(19)
chopes<-grep("chopes", FabianaB, ignore.case=T, value=F)
chopes 

#(21)
FabianaB.falantes<-gsub("D1","Doc",FabianaB)
FabianaB.falantes<-gsub("S1","Inf",FabianaB.falantes)
FabianaB.falantes

#(23)
cat(FabianaB.falantes, file = "FabianaB-part.txt", sep="\n")

#(26)
MauricioB<-scan(file=choose.files(),what="char",sep="")
MauricioB

#(29)
grep("\\b.+[e][nm][bcdfgjklpqrstvxz]", MauricioB, value=T)

#(31)
exemplo.R<-c("na 3, 4 srie, eu era muito bagunceira",
  "da ele gritou assim... abre a PO::::rta!",
  "eles vo e [barulho de porta batendo] fazem",
  "eu num vou fal(r) p(r)a ele que ele (es)t errado n?")

#(32)
grep("\\d", exemplo.R, value=T) 

#(33)
exemplo.R1<-gsub(":|\\(|\\)|\\[.+\\]", "", exemplo.R)
exemplo.R2<-gsub("3", "terceira", exemplo.R1)
exemplo.R3<-gsub("4", "quarta", exemplo.R2)
exemplo.R3

#(34)
source("~/dmsocio.R")

#(35)
source("http://tinyurl.com/dmsocio")
setwd("~/dmsocio-SP2010") 

#(36)
identificacao(padrao="inh[oa]s?\\b", posicao.marcacao=T)

#(37)
ignorar<-c("D1", "S2", "S3", "S4", "S5", "Dados Contextuais", "Roteiro","Universidade de", "Faculdade de", "Departamento de", "Grupo de", "Projeto SP2010", "(FAPESP)", "Documentador:", "Informante:", "Arquivos:", "Perfil:", "Sexo/Gnero:", "Idade:", "Escolaridade:", "Regio:", "Zona:", "Gerao:", "Renda individual:", "Renda familiar:")

#(38)
pal.stoplist<-c("vizinh[oa]s?", "minha", "tinha", "carinho")

#(39)
identificacao(padrao="inh[oa]s?\\b",
              posicao.marcacao=T,
              ignorar.linhas=ignorar,
              stoplist=pal.stoplist)

#(41)
identificacao(padrao="[aeiou]r[bcdfgjklmnpqstvwxz].*?\\b",
              simbolo.marcacao="<>",
              posicao.marcacao=T,
              ignorar.linhas=ignorar,
              stoplist=NULL,
              novos.arquivos="marcacoes")

#(43)
identificacao(padrao="[aeiou]r[bcdfgjklmnpqstvwxz].*?
              [aeiou]r[bcdfgjklmnpqstvwxz].*?\\b",
              simbolo.marcacao="<>",
              posicao.marcacao=T,
              ignorar.linhas=ignorar,
              stoplist=NULL,
              novos.arquivos="marcacoes")

#(44)
identificacao(padrao="[aeiou]r\\b",
              simbolo.marcacao="<>",
              posicao.marcacao=T,
              ignorar.linhas=ignorar,
              stoplist=NULL,
              novos.arquivos="marcacoes")

#(45)
identificacao(padrao="ns\\b|\\bnos\\b|conosco|noss[oa]s?\\b",
              simbolo.marcacao="<N>",
              posicao.marcacao=T,
              ignorar.linhas=ignorar,
              stoplist=NULL,
              novos.arquivos="marcacoes")


#(46)
stoplist.G<-c("muita\\sgente","tanta\\sgente","pouca\\sgente")
identificacao(padrao="a\\sgente",
              simbolo.marcacao="<G>",
              posicao.marcacao=T,
              ignorar.linhas=ignorar,
              stoplist=stoplist.G,
              novos.arquivos="marcacoes")

#(47)
extracao("inh[oa]s?\\b")

#(48)
var.sociais<-c(3, 5, 6, 7, 12) 

#(49)
nomes.var.sociais<-c("sexo.genero", "faixa.etaria", "escolaridade", "regiao.residencia", "informante")

#(50)
setwd("~/dmsocio-Codif-R")
          
extracao(padrao="<T>|<R>",
         palavras.cont.precedente=4,
         palavras.ocorrencia=2,
         palavras.cont.seguinte=4,
         stoplist=NULL,
         ignorar.linhas=NULL,
         var.dependente=2,
         loc.variaveis.sociais=var.sociais, 
         nomes.colunas.variaveis=nomes.var.sociais,
         file="DadosExtraidos-RT.txt")

#(51)
dadosR<-read.table(file=choose.files(), header=T, sep="\t", quote="", comment.char="")

#(52)
amostragem(numero.dados=100,
           coluna.referencia=9,
           data=dadosR,
           novo.arquivo="DadosAmostrados-RT.txt")

